/* This file can be redistributed and used under GNU Library General Public
 * License. (c) 2000 Micha Moskal <malekith@topnet.pl>. 
 * $Id: msgbox.cc,v 1.4 2000/09/09 17:59:14 malekith Exp $ */

#include <v/v.h>
#include <v/term.h>


#define V_MSGBOX_BASE (ev_user + 100)

class MessageBox : public Dialog {
protected:
	void set_sizes(int blen);
	int add_buttons();
	void move_buttons(int blen);
public:
	virtual int execute();
	String text, buttons;
};

void MessageBox::set_sizes(int blen)
{
	int min_w = title.strlen() + 6;
	TextArea *a = new TextArea;
	String dummy;
	
	add_child(a);
	
	blen += 4;
	
	if (min_w < blen)
		min_w = blen;
		
	a->caption = text;
	
	if (min_w > s_width - 4)
		min_w = s_width - 4;
	else
		for (int k = a->word_break(min_w, dummy);
		     min_w < s_width - 4 && 
		     	     (k < 0 || k > s_height - 4 || k + 4 > min_w / 2); 
		     min_w++)
			k = a->word_break(min_w, dummy);
	
	a->pos.w = min_w;
	a->pos.h = a->word_break(min_w, dummy);
	a->pos.move_to(2,1);
	
	pos.resize_to(a->pos.w + 4, a->pos.h + 4);
	center();
}

int MessageBox::add_buttons()
{
	char *b = strdup(buttons);
	char *p;
	int ev;
	int len = -1;

	for (p = b, ev = V_MSGBOX_BASE; p && *p; ev++) {
		char *np;
		CommandButton *cb;
		
		if ((np = strchr(p, '|')))
			*np++ = 0;
			
		cb = new CommandButton;
		cb->command = ev;
		cb->set_caption(p);
		
		add_child(cb);
		
		len += cb->pos.w + 1;
		
		p = np;
	}
	free(b);
	return len;
}

void MessageBox::move_buttons(int blen)
{
	int x = (pos.w - blen) / 2, y = pos.h - 2;
	for (Widget *w = children; w; w = w->next)
		if (w->pos.x == 0) {
			w->pos.move_to(x, y);
			x += w->pos.w + 1;
		}
}

int MessageBox::execute()
{
	int w;
	w = add_buttons();
	set_sizes(w);
	move_buttons(w);
	Dialog::execute();
	return modal_result ? modal_result - V_MSGBOX_BASE + 1 : 0;
}

int v_message_box(const String &text, const String &title, 
		  const String &buttons)
{
	MessageBox *m;
	
	m = new MessageBox;
	m->title = title;
	m->text = text;
	m->buttons = buttons;
	
	return execute_dialog(m);
}

